/*  smplayer, GUI front-end for mplayer.
    Copyright (C) 2006-2012 Ricardo Villalba <rvm@users.sourceforge.net>
    Copyright (C) 2010 Ori Rejwan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "retrieveyoutubeurl.h"
#include <QUrl>
#include <QDebug>

RetrieveYoutubeUrl::RetrieveYoutubeUrl( QObject* parent ) : SimpleHttp(parent)
{
    connect(this, SIGNAL(downloadFinished(QByteArray)),
            this, SLOT(parse(QByteArray)));

    preferred_quality = VQ_720p;
}

RetrieveYoutubeUrl::~RetrieveYoutubeUrl() {
}

void RetrieveYoutubeUrl::fetchPage(const QString & url) {
    download(url);
    orig_url = url;
}

QString RetrieveYoutubeUrl::findVideoType(const QMap<int, QString> &urlMap, const QString &url)
{
    int q = urlMap.key(url, MP4_720p);

    if (q == FLV_240p || q == FLV_360p || q == FLV_480p)
        return QString("flv");
    if (q == MP4_360p || q == MP4_720p || q == MP4_1080p)
        return QString("mp4");
    if (q == WEBM_360p || q == WEBM_480p || q == WEBM_720p || q == WEBM_1080p)
        return QString("webm");
    else
        return QString();
}

void RetrieveYoutubeUrl::parse(QByteArray text) {
    //qDebug("RetrieveYoutubeUrl::parse");

    urlMap.clear();

    QString replyString = QString::fromUtf8(text.constData(), text.size());

    QRegExp rx_title(".*<title>(.*)</title>.*");
    if (rx_title.indexIn(replyString) != -1) {
        url_title = rx_title.cap(1).simplified();
        url_title = QString(url_title).replace("&amp;","&").replace("&gt;", ">").replace("&lt;", "<").replace("&quot;","\"").replace("&#39;","'")/*.replace(" - YouTube", "")*/;
        ////qDebug("RetrieveYoutubeUrl::parse: title '%s'", url_title.toUtf8().constData());
    } else {
        url_title = "Youtube video";
    }

    QRegExp regex("\\\"url_encoded_fmt_stream_map\\\"\\s*:\\s*\\\"([^\\\"]*)");
    regex.indexIn(replyString);
    QString fmtArray = regex.cap(1);
    fmtArray = sanitizeForUnicodePoint(fmtArray);
    fmtArray.replace(QRegExp("\\\\(.)"), "\\1");
    htmlDecode(fmtArray);
    QStringList codeList = fmtArray.split(',');
    foreach(QString code, codeList)
    {
        // (2012-12-20) Youtube Fix by RVM for SMPlayer (http://smplayer.sourceforge.net)
        int p = code.indexOf("url=");
        if (p > -1) {
            QString u = code.mid(p+4);
            QString l = code.left(p);
            p = l.indexOf("sig=");
            if (p > -1) {
                QString sig = l.mid(p+4);
                p = sig.indexOf("&");
                if (p > -1) sig = sig.left(p);
//                qDebug("RetrieveYoutubeUrl::parse: sig: %s", sig.toLatin1().constData());
                u += "&sig=" + sig;
            }
//            qDebug("RetrieveYoutubeUrl::parse: l: '%s'", l.toLatin1().constData());
            QUrl url(u);
            int itag = url.queryItemValue("itag").toInt();
//            qDebug("RetrieveYoutubeUrl::parse: itag: %d", itag);
            u.replace("&sig=", "&signature=");
            QRegExp rx("&itag=(\\d+)");
            p = u.indexOf(rx);
//            qDebug("RetrieveYoutubeUrl::parse: p: %d", p);
            if (p > -1) {
                p = u.indexOf(rx, p+1);
                if (p > -1) {
                    QString s = "&itag=" + rx.cap(1);
//                    qDebug("RetrieveYoutubeUrl::parse: **** itag: %s", s.toLatin1().constData());
                    u.replace(p, s.length(), "");
                }
            }
            urlMap[itag] = u;
        }
    }

    //qDebug("RetrieveYoutubeUrl::parse: url count: %d", urlMap.count());

    QString p_url = findPreferredUrl();
    if (!p_url.isNull()) {
        emit gotUrls(urlMap);
        emit gotPreferredUrl(p_url);
        if (!latest_video_type.isEmpty())
            emit gotVideoType(latest_video_type);
    } else {
        emit gotEmptyList();
    }
}

QString RetrieveYoutubeUrl::findPreferredUrl() {
    latest_preferred_url = findPreferredUrl(urlMap, preferred_quality);
    latest_video_type = findVideoType(urlMap, latest_preferred_url);
    return latest_preferred_url;
}

QString RetrieveYoutubeUrl::findPreferredUrl(const QMap<int, QString>& urlMap, PreferredQuality quality)
{
    QMap<PreferredQuality, Quality> quality_map;

    quality_map.insertMulti(VQ_240p, FLV_240p);
    quality_map.insertMulti(VQ_360p, FLV_360p);
    quality_map.insertMulti(VQ_360p, WEBM_360p);
    quality_map.insertMulti(VQ_480p, FLV_480p);
    quality_map.insertMulti(VQ_480p, WEBM_480p);
    quality_map.insertMulti(VQ_720p, WEBM_720p);
    quality_map.insertMulti(VQ_1080p, WEBM_1080p);

    Quality q = MP4_720p;
    QList<Quality> qualities = quality_map.values(quality);
    if (qualities.size() != 0) {
        q = qualities.last();
    }

    // Choose a url according to preferred quality
    QString p_url;

    if (q==WEBM_1080p) {
        p_url = urlMap.value(WEBM_1080p, QString());
        if (p_url.isNull()) q = WEBM_720p;
    }

    if (q==WEBM_720p) {
        p_url = urlMap.value(WEBM_720p, QString());
        if (p_url.isNull()) q = FLV_480p;
    }

    if (q==FLV_480p) {
        p_url = urlMap.value(FLV_480p, QString());
        if (p_url.isNull()) p_url = urlMap.value(WEBM_480p, QString());
        if (p_url.isNull()) q = FLV_360p;
    }

    if (q==FLV_360p) {
        p_url = urlMap.value(FLV_360p, QString());
        if (p_url.isNull()) p_url = urlMap.value(WEBM_360p, QString());
        if (p_url.isNull()) q = FLV_240p;
    }

    // WEBM, low priority
    if (q==WEBM_480p) {
        p_url = urlMap.value(WEBM_480p, QString());
        if (p_url.isNull()) p_url = urlMap.value(FLV_360p, QString());
        if (p_url.isNull()) q = WEBM_360p;
    }

    if (q==WEBM_360p) {
        p_url = urlMap.value(WEBM_360p, QString());
        if (p_url.isNull()) q = FLV_240p;
    }

    if (q==FLV_240p) {
        p_url = urlMap.value(q, QString());
    }

    return p_url;
}

QString RetrieveYoutubeUrl::sanitizeForUnicodePoint(QString string)
{
    QRegExp rx("\\\\u(\\d{4})");
    while (rx.indexIn(string) != -1) {
        string.replace(rx.cap(0), QString(QChar(rx.cap(1).toInt(0,16))));
    }
    return string;
}

void RetrieveYoutubeUrl::htmlDecode(QString& string)
{
    string.replace("%3A", ":", Qt::CaseInsensitive);
    string.replace("%2F", "/", Qt::CaseInsensitive);
    string.replace("%3F", "?", Qt::CaseInsensitive);
    string.replace("%3D", "=", Qt::CaseInsensitive);
    string.replace("%25", "%", Qt::CaseInsensitive);
    string.replace("%26", "&", Qt::CaseInsensitive);
    string.replace("%3D", "=", Qt::CaseInsensitive);
}

#include "moc_retrieveyoutubeurl.cpp"
